import mset, os, json, urllib

#version 3.2.1

def checkAllObjects(objectName):
    obj = mset.findObject(objectName)
    if obj is not None:
        obj.destroy()
        checkAllObjects(objectName)

def CheckIsBillboard(assetJson):
    if(assetJson['type'] == "3dplant"):
        if (assetJson['activeLOD'] == assetJson['minLOD']):
            return True
    return False


#the main import function
def startImport():

    try:
        from urllib.request import urlopen
        response = urlopen('http://localhost:28241/getLastExport/')
        jsonFile = response.read().decode('utf-8')

        if jsonFile is None or jsonFile == "":
            return

        json_Array = json.loads(jsonFile)

        if(len(json_Array) > 0):
            for js in json_Array:
                
                texturesToUse = 'components'

                isPlant = True if (js['type'].find('plant')) else False
                if(isPlant): 
                    if(CheckIsBillboard(js)):
                        texturesToUse = 'components-billboard'

                n = (js['name'] + "_" + js['id']).replace(" ", "_")

                mat = mset.importMaterial(os.path.join(os.getcwd(), 'data', 'plugin', 'MSLiveLink', 'QxlMat.tbmat'), n)
                gFound = False
                checkAllObjects(n)

                textureTypes = [obj["type"] for obj in js[texturesToUse]]

                for texComp in js[texturesToUse]:
                    pth = texComp['path']
                    texType = texComp['type']
                    tex = mset.Texture(pth)

                    if texType == 'albedo':
                        mat.albedo.setField('Albedo Map', tex )
                        tex.sRGB = True

                    elif texType == 'diffuse' and "albedo" not in textureTypes:
                        mat.albedo.setField('Albedo Map', tex )
                        tex.sRGB = True

                    elif texType == 'translucency':
                        mat.setSubroutine('diffusion', 'Subsurface Scatter')
                        mat.diffusion.setField('Translucency Map', tex)
                        tex.sRGB = True
                        mat.diffusion.setField('Translucency', 1.0)
                        mat.diffusion.setField('Translucency Depth (mm)', 20.0)
                        mat.diffusion.setField('Translucency Scatter', 0.35)
                        mat.diffusion.setField('Translucency Color', [1.0,1.0,1.0])
                        mat.diffusion.setField('Scatter Color', [1.0,1.0,1.0])

                    elif texType == 'opacity':
                        mat.setSubroutine('transparency', 'Dither')
                        mat.transparency.setField('Alpha Map', tex)
                        mat.transparency.setField('Use Albedo Alpha', False)
                        mat.transparency.setField('Channel', 0)

                    elif texType == 'AO':
                        mat.setSubroutine('occlusion', 'Occlusion')
                        mat.occlusion.setField('Occlusion Map', tex)

                    elif texType == 'cavity':
                        mat.setSubroutine('occlusion', 'Occlusion')
                        mat.occlusion.setField('Cavity Map', tex )
                        mat.occlusion.setField('Diffuse Cavity', 0.0)

                    elif texType == 'displacement':
                        mat.setSubroutine('subdivision', 'Flat')
                        mat.subdivision.setField('Tessellation', 256)
                        mat.setSubroutine('displacement', 'Height')
                        mat.displacement.setField('Scale', 0.025)
                        mat.displacement.setField('Displacement Map', tex )

                    elif texType == 'gloss':
                        mat.microsurface.setField('Gloss Map', tex )
                        mat.microsurface.setField('Invert', False)
                        gFound = True

                    elif texType == 'normal':
                        mat.surface.setField('Normal Map', tex )

                    elif texType == 'roughness' and gFound == False:
                        mat.setSubroutine('microsurface', 'Roughness')
                        mat.microsurface.setField('Roughness Map', tex )

                    elif texType == 'metalness':
                        mat.setSubroutine('reflectivity', 'Metalness')
                        mat.reflectivity.setField('Metalness Map', tex )

                    elif texType == 'specular':
                        mat.setSubroutine('reflectivity', 'Specular')
                        mat.reflectivity.setField('Specular Map', tex )
                        tex.sRGB = True

                for meshComp in js['meshList']:
                    pth = meshComp['path']
                    mesh = mset.importModel(pth)
                    mesh.name = n
                    mc = mesh.getChildren()
                    for c in mc:
                        if isPlant:
                            c.cullBackFaces = False
                    mat.assign(mesh, True)
            
            # response = urlopen('http://localhost:28241/clearLastExport/')
    except Exception as e:
        print(e)
        pass

#create a window
msWindow = mset.UIWindow("MSPlugin")
msWindow.width = 76

#the button that starts the import process.
msButton = mset.UIButton()
msButton.onClick = startImport
msButton.setIcon(os.path.join(os.getcwd(), 'data', 'plugin', 'MSLiveLink', 'M.png'))
msWindow.addElement(msButton)

msWindow.addStretchSpace()
msWindow.addReturn()

close_win = mset.UIButton("   Close")
close_win.onClick = mset.shutdownPlugin
msWindow.addElement(close_win)